#
# spec file for package supergfxctl
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global rustflags -Clink-arg=-Wl,-z,relro,-z,now

Name:           supergfxctl
Version:        5.2.7
Release:        1.3
Summary:        Super graphics mode controller
License:        MPL-2.0
URL:            https://gitlab.com/asus-linux/supergfxctl
Source0:        %{name}-%{version}.tar.gz
Source1:        vendor.tar.zst
Source2:        %{name}-user.conf
Source3:        prime-run.sh
Group:          System/Daemons

## Patch for user-groups
Patch1:         user-group.patch
##

BuildRequires:  cargo-packaging
BuildRequires:  pkgconfig
BuildRequires:  sysuser-tools
BuildRequires:  pkgconfig(dbus-1)
BuildRequires:  pkgconfig(libudev)
BuildRequires:  pkgconfig(systemd)
Requires:       systemd

Conflicts:      bbswitch
Conflicts:      bumblebee
Conflicts:      bumblebee-status-module-nvidia-optimus-manager
Conflicts:      suse-prime
Conflicts:      system76-power

%sysusers_requires

%description
supergfxctl is a super graphics mode controller for laptops with hybrid nvidia.

%prep
%autosetup -a1 -p1

%build
%cargo_build
%sysusers_generate_pre %{SOURCE2} %{name} %{name}-user.conf

%install
%cargo_install

mkdir -p "%{buildroot}%{_bindir}"
install -D -m 0755 target/release/supergfxd %{buildroot}%{_bindir}/supergfxd
install -D -m 0755 target/release/supergfxctl %{buildroot}%{_bindir}/supergfxctl
install -D -m 0644 data/90-supergfxd-nvidia-pm.rules %{buildroot}%{_udevrulesdir}/90-supergfxd-nvidia-pm.rules
install -D -m 0644 data/org.supergfxctl.Daemon.conf  %{buildroot}%{_sysconfdir}/dbus-1/system.d/org.supergfxctl.Daemon.conf
install -D -m 0644 data/supergfxd.service %{buildroot}%{_unitdir}/supergfxd.service
install -D -m 0644 data/supergfxd.preset %{buildroot}%{_presetdir}/99-supergfxd.preset

install -D -m 0644 LICENSE %{buildroot}%{_datadir}/licenses/%{name}/LICENSE
install -D -m 0644 README.md %{buildroot}%{_datadir}/doc/%{name}/README.md

install -D -m 0644 %{SOURCE2} %{buildroot}%{_sysusersdir}/%{name}-user.conf

install -D -m 0755 %{SOURCE3} %{buildroot}%{_bindir}/prime-run

%pre -f %{name}.pre
%service_add_pre supergfxd.service

%post
%service_add_post supergfxd.service

%preun
%service_del_preun supergfxd.service

%postun
%service_del_postun_with_restart supergfxd.service
if [ -e %{_sysconfdir}/modprobe.d/nvidia.conf ]
then
    if grep -q "# options nvidia-drm.modeset=1" %{_sysconfdir}/modprobe.d/nvidia.conf
    then
        echo "optional nvidia-drm.modeset=1" > %{_sysconfdir}/modprobe.d/nvidia.conf
    fi
fi

%files
%license LICENSE
%{_bindir}/supergfxd
%{_bindir}/supergfxctl
%{_bindir}/prime-run
%{_unitdir}/supergfxd.service
%{_presetdir}/99-supergfxd.preset
%{_udevrulesdir}/90-supergfxd-nvidia-pm.rules
%{_sysusersdir}/%{name}-user.conf
%ghost %attr(0644,root,root) %{_sysconfdir}/modprobe.d/supergfxd.conf
%config %{_sysconfdir}/dbus-1/system.d/org.supergfxctl.Daemon.conf
%dir %{_datadir}/doc/%{name}
%{_datadir}/doc/%{name}/*

%changelog
* Sun Feb 16 2025 RN <R_Nik_C@proton.me>
- Update to version 5.2.7:
  * Various fixes. Use display port method to find gpu
  * Bump zbus and version
  * Update deps
  * Remove unnecessary debug info
  * For loop no longer ends when it encounters an error while
    reading the hwmon directory.
  * Update CHANGELOG.md
  * Fix dGPU detection for FA401WV
  * Bump version and deps
- Superseded GPUDetection.patch
* Fri Nov 22 2024 RN <R_Nik_C@proton.me>
- Added prime-run binary that just links process to the nvidia
  libraries
* Mon Nov 18 2024 RN <R_Nik_C@proton.me>
- Added upstream GPU detection patch
* Mon Nov 18 2024 RN <R_Nik_C@proton.me>
- Added patch for user groups
- Removed `users`, `sudo`, `adm` and `wheel` groups
- Added `video` group and separated `supergfxctl`
* Thu Oct  3 2024 RN <R_Nik_C@proton.me>
- Update to version 5.2.4 release:
  * Update CHANGELOG.md
  * Fix dGPU detection for FA401WV
* Sun Sep 22 2024 RN <R_Nik_C@proton.me>
- Update to version 5.2.4:
  * Bump version and deps
  * fix: check alternate path for egpu_enable flag
  * Bump deps and version
  * Bump deps
  * Remove the nvidia/false config stuff as too problematic
  * Further tweaks to asus dgpu safety check
  * asus egpu fixes
  * Update deps
  * Update changelog
* Wed Jun 12 2024 RN <R_Nik_c@proton.me>
- Fixing update process bug
* Thu Mar 28 2024 RN <R_Nik_c@proton.me>
- Update to 5.2.1
- Better sanity check for booting without egpu after it was previously set
- Added Vulkan ICD profile switching
* Thu Dec 28 2023 RN <R_Nik_C@proton.me>
- Switched to _service usage, because cargo_config is part of the vendor.tar now
- Cleaned up .spec
* Wed Dec 27 2023 Jan Engelhardt <jengelh@inai.de>
- Avoid bashisms in build recipe
* Tue Nov 14 2023 RN <R_Nik_C@proton.me>
- Update to version 5.1.2:
  * Bump deps
  * Add missing entry in changelog
  * Adjust makefile to use vender-filter
  * Ensure supergfx tries to fallback if change fails
  * Small fixes for egpu
  * Bump version
  * Update deps
  * Update deps
  * Fix: Don't continue with action list if the error is a logout timeout
  * Fix pipeline
* Fri Nov 18 2022 luke@ljones.dev
- Update to version 5.0.1:
  * Add dgpu status notification on dbus
  * Fix: Check for GA401I, remove compute mode, set log level info
  * Release 5.0.0
  * Prep tag RC15
  * RC15, final
  * RC13 with various corrections
  * Update dependencies
  * Enable hotplug use, fixes to dgpu_disable, fixes to integrated
  * Do pci rescan even if no missing device
  * Update changelog
  * Switch to async with tokio. Use asus switches if avilable. Bugfixes.
  * Ignore possible i/o error as invalid
  * Bump zbus version
  * Add 99-nvidia-ac.rules udev rule
  * fix: correction in changelog
  * fix: make sure to use asus_use_dgpu_disable
  * Add pipeline, cleanup with clippy
  * 5.0.0-rc1
  * Update dbus API docs (introspectable)
  * Add more debug logging, default service to debug for now
  * Switch to async runtime
  * Slap Arc<Mutex<>> around gpu device
  * Switch to udev. Add ASUS methods. Optimise.
  * Remove dedicated mode
  * Added pipe to sh command to install Rust Taken from the official Rust website -- https://www.rust-lang.org/tools/install
  * fix AMD detection for pci-ids version 2020.03.20
* Tue Jul  5 2022 luke@ljones.dev
- Update to version 4.0.5:
  * Prep new release
  * Fix interaction with newer lspci version
  * add "Quadro" to the lspci parsing for NVIDIA cards
* Wed May 11 2022 Enrico Belleri <idesmi@protonmail.com>
- Use obs services for source and vendor
* Fri Feb  4 2022 Luke Jones <luke@ljones.dev>
- Version 4.0.4
- Adjust the kernel cmdline arg code path
* Thu Feb  3 2022 Luke Jones <luke@ljones.dev>
- Version 4.0.3
- Add config option `no_logind`: Don't use logind to see if all sessions are
  logged out and therefore safe to change mode. This will be useful for people not
  using a login manager, however it is not guaranteed to work unless all graphical
  sessions are ended and nothing is hooking the drivers. Ignored if `always_reboot`
  is set.
- Add config option `logout_timeout_s`: The timeout in seconds to wait for all user
  graphical sessions to end. Default is 3 minutes, 0 = infinite. Ignored if
  `no_logind` or `always_reboot` is set.
- Add new dbus method: `PendingMode`, to check if a mode change is required
- Add new dbus method: `PendingUserAction`, to check if the user is required to perform an action
- Add new dbus method: `Config`, to get the current base config
- Add new dbus method: `SetConfig`, to set the base config
- Add `-p, --pend-action` CLI arg to get the pending user action if any
- Add `-P, --pend-mode` CLI arg to get the pending mode change if any`
- Add ability to read `supergfxd.mode=` from kernel cmdline on startup and set the mode appropriately
- CLI option `--force` was unused, it is now removed.
* Sat Jan 22 2022 Luke Jones <luke@ljones.dev>
- Version 4.0.2
* Wed Jan 19 2022 Luke Jones <luke@ljones.dev>
- Version 4.0.1
- Fix version upgrade of config
- Recreate the config if parsing fails
- Only write the (reboot) mode change to config file, don't update live config
- AMD dedicated + hybrid config for xorg
- "AllowExternalGpus" added to xorg for Nvidia Egpu mode
* Tue Jan 18 2022 Luke Jones <luke@ljones.dev>
- Version 4.0.0
- Added
  - Add new dbus method: `Version` to get supergfxd version
  - Add new dbus method: `Vendor` to get dGPU vendor name
  - Add new dbus method: `Supported` to get list of supported modes
  - Add `-v, --version` CLI arg to get supergfxd version
  - Add `-V, --vendor` CLI arg to get dGPU vendor name
  - Add `-s, --supported` CLI arg to get list of supported modes
  - Add new config option: `vfio_save` to reload VFIO on boot
  - Add new config option: `compute_save` to reload compute on boot
  - Add new config option: `always_reboot` reboot to change modes
- Changed
  - Adjust startup to check for ASUS eGPU and dGPU enablement if no modes supported
  - If nvidia-drm.modeset=1 is set then save mode and require a reboot by default\
  - Add extra check for Nvidia dGPU (fixes Flow 13")
  - Properly check the correct device for power status
- Breaking
  - Rename Vendor, GetVendor to Mode, GetMode to better reflect their results
* Mon Jan 10 2022 Luke Jones <luke@ljones.dev>
- Version 3.0.0
* Sun Dec 19 2021 Luke Jones <luke@ljones.dev>
- Version 2.0.5
* Tue Nov  2 2021 Luke Jones <luke@ljones.dev>
- Version 2.0.4
* Tue Nov  2 2021 Luke Jones <luke@ljones.dev>
- Version 2.0.3
